@@ -1,4 +1,4 @@
-# Copyright 2014-2018 Facundo Batista, Nicolás Demarchi
+# Copyright 2014-2020 Facundo Batista, Nicolás Demarchi
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -38,7 +38,7 @@ PIP_INSTALLER = "https://bootstrap.pypa.io/get-pip.py"
 class PipManager():
     """A manager for all PIP related actions."""
 
-    def __init__(self, env_bin_path, pip_installed=False, options=None):
+    def __init__(self, env_bin_path, pip_installed=False, options=None, avoid_pip_upgrade=False):
         """Init."""
         self.env_bin_path = env_bin_path
         self.pip_installed = pip_installed
@@ -46,6 +46,7 @@ class PipManager():
         self.pip_exe = os.path.join(self.env_bin_path, "pip")
         basedir = helpers.get_basedir()
         self.pip_installer_fname = os.path.join(basedir, "get-pip.py")
+        self.avoid_pip_upgrade = avoid_pip_upgrade
 
     def install(self, dependency):
         """Install a new dependency."""
@@ -54,6 +55,12 @@ class PipManager():
                         "doing it manually (just wait a little, all should go well)")
             self._brute_force_install_pip()
 
+        # Always update pip to get latest behaviours (specially regarding security); this has
+        # the nice side effect of getting logged the pip version that is used.
+        if not self.avoid_pip_upgrade:
+            python_exe = os.path.join(self.env_bin_path, "python")
+            helpers.logged_exec([python_exe, '-m', 'pip', 'install', 'pip', '--upgrade'])
+
         # split to pass several tokens on multiword dependency (this is very specific for '-e' on
         # external requirements, but implemented generically; note that this does not apply for
         # normal reqs, because even if it originally is 'foo > 1.2', after parsing it loses the
@@ -108,3 +115,10 @@ class PipManager():
         python_exe = os.path.join(self.env_bin_path, "python")
         helpers.logged_exec([python_exe, self.pip_installer_fname, '-I'])
         self.pip_installed = True
+
+    def freeze(self, filepath):
+        """Dump venv contents to the indicated filepath."""
+        logger.debug("running freeze to store in %r", filepath)
+        stdout = helpers.logged_exec([self.pip_exe, "freeze", "--all", "--local"])
+        with open(filepath, "wt", encoding='utf8') as fh:
+            fh.writelines(line + '\n' for line in sorted(stdout))
