@@ -21,7 +21,7 @@ import shutil
 import tempfile
 import unittest
 from datetime import datetime, timedelta
-from unittest.mock import Mock, patch
+from unittest.mock import Mock, patch, call
 
 from pkg_resources import parse_requirements
 
@@ -65,6 +65,7 @@ class EnvCreationTestCase(unittest.TestCase):
         }
         interpreter = 'python3'
         is_current = True
+        avoid_pip_upgrade = False
         options = {"virtualenv_options": [],
                    "pyvenv_options": [],
                    }
@@ -74,8 +75,8 @@ class EnvCreationTestCase(unittest.TestCase):
                 mock_create.return_value = ('env_path', 'env_bin_path', 'pip_installed')
                 mock_mgr_c.return_value = fake_manager = self.FakeManager()
                 fake_manager.really_installed = {'dep1': 'v1', 'dep2': 'v2'}
-                venv_data, installed = envbuilder.create_venv(requested, interpreter, is_current,
-                                                              options, pip_options)
+                venv_data, installed = envbuilder.create_venv(
+                    requested, interpreter, is_current, options, pip_options, avoid_pip_upgrade)
 
         self.assertEqual(venv_data, {
             'env_bin_path': 'env_bin_path',
@@ -88,6 +89,10 @@ class EnvCreationTestCase(unittest.TestCase):
                 'dep2': 'v2',
             }
         })
+        expected_pipmanager_call = call(
+            'env_bin_path', pip_installed='pip_installed', options=[],
+            avoid_pip_upgrade=avoid_pip_upgrade)
+        self.assertEqual(mock_mgr_c.call_args, expected_pipmanager_call)
 
     def test_create_vcs(self):
         requested = {
@@ -95,14 +100,15 @@ class EnvCreationTestCase(unittest.TestCase):
         }
         interpreter = 'python3'
         is_current = True
+        avoid_pip_upgrade = False
         options = {"virtualenv_options": [], "pyvenv_options": []}
         pip_options = []
         with patch.object(envbuilder._FadesEnvBuilder, 'create_env') as mock_create:
             with patch.object(envbuilder, 'PipManager') as mock_mgr_c:
                 mock_create.return_value = ('env_path', 'env_bin_path', 'pip_installed')
                 mock_mgr_c.return_value = self.FakeManager()
-                venv_data, installed = envbuilder.create_venv(requested, interpreter, is_current,
-                                                              options, pip_options)
+                venv_data, installed = envbuilder.create_venv(
+                    requested, interpreter, is_current, options, pip_options, avoid_pip_upgrade)
 
         self.assertEqual(venv_data, {
             'env_bin_path': 'env_bin_path',
@@ -117,6 +123,7 @@ class EnvCreationTestCase(unittest.TestCase):
         }
         interpreter = 'python3'
         is_current = True
+        avoid_pip_upgrade = False
         options = {"virtualenv_options": [],
                    "pyvenv_options": [],
                    }
@@ -125,7 +132,8 @@ class EnvCreationTestCase(unittest.TestCase):
             with patch.object(envbuilder, 'PipManager') as mock_mgr_c:
                 mock_create.return_value = ('env_path', 'env_bin_path', 'pip_installed')
                 mock_mgr_c.return_value = self.FakeManager()
-                envbuilder.create_venv(requested, interpreter, is_current, options, pip_options)
+                envbuilder.create_venv(
+                    requested, interpreter, is_current, options, pip_options, avoid_pip_upgrade)
 
         self.assertLoggedWarning("Install from 'unknown' not implemented")
 
@@ -135,6 +143,7 @@ class EnvCreationTestCase(unittest.TestCase):
         }
         interpreter = 'python3'
         is_current = True
+        avoid_pip_upgrade = False
         options = {'virtualenv_options': [],
                    'pyvenv_options': [],
                    }
@@ -147,11 +156,8 @@ class EnvCreationTestCase(unittest.TestCase):
                 with patch.object(envbuilder, 'destroy_venv', spec=True) as mock_destroy:
                     with self.assertRaises(FadesError) as cm:
                         envbuilder.create_venv(
-                            requested,
-                            interpreter,
-                            is_current,
-                            options,
-                            pip_options)
+                            requested, interpreter, is_current, options, pip_options,
+                            avoid_pip_upgrade)
                     self.assertEqual(str(cm.exception), 'Dependency installation failed')
                     mock_destroy.assert_called_once_with('env_path')
 
@@ -163,6 +169,7 @@ class EnvCreationTestCase(unittest.TestCase):
         }
         interpreter = 'python3'
         is_current = True
+        avoid_pip_upgrade = False
         options = {"virtualenv_options": [],
                    "pyvenv_options": [],
                    }
@@ -172,8 +179,8 @@ class EnvCreationTestCase(unittest.TestCase):
                 mock_create.return_value = ('env_path', 'env_bin_path', 'pip_installed')
                 mock_mgr_c.return_value = fake_manager = self.FakeManager()
                 fake_manager.really_installed = {'dep1': 'vX', 'dep2': 'v2'}
-                _, installed = envbuilder.create_venv(requested, interpreter, is_current, options,
-                                                      pip_options)
+                _, installed = envbuilder.create_venv(
+                    requested, interpreter, is_current, options, pip_options, avoid_pip_upgrade)
 
         self.assertEqual(installed, {
             REPO_PYPI: {
@@ -190,9 +197,9 @@ class EnvCreationTestCase(unittest.TestCase):
                    "pyvenv_options": ['--system-site-packages'],
                    }
         with patch.object(EnvBuilder, 'create') as mock_create:
-                env_builder.create_env(interpreter, is_current, options)
-                self.assertTrue(env_builder.system_site_packages)
-                self.assertTrue(mock_create.called)
+            env_builder.create_env(interpreter, is_current, options)
+            self.assertTrue(env_builder.system_site_packages)
+            self.assertTrue(mock_create.called)
 
     def test_create_pyvenv(self):
         env_builder = envbuilder._FadesEnvBuilder()
@@ -202,9 +209,9 @@ class EnvCreationTestCase(unittest.TestCase):
                    "pyvenv_options": [],
                    }
         with patch.object(EnvBuilder, 'create') as mock_create:
-                env_builder.create_env(interpreter, is_current, options)
-                self.assertFalse(env_builder.system_site_packages)
-                self.assertTrue(mock_create.called)
+            env_builder.create_env(interpreter, is_current, options)
+            self.assertFalse(env_builder.system_site_packages)
+            self.assertTrue(mock_create.called)
 
     def test_create_system_site_pkgs_virtualenv(self):
         env_builder = envbuilder._FadesEnvBuilder()
@@ -214,8 +221,8 @@ class EnvCreationTestCase(unittest.TestCase):
                    "pyvenv_options": [],
                    }
         with patch.object(envbuilder._FadesEnvBuilder, 'create_with_virtualenv') as mock_create:
-                env_builder.create_env(interpreter, is_current, options)
-                mock_create.assert_called_with(interpreter, options['virtualenv_options'])
+            env_builder.create_env(interpreter, is_current, options)
+            mock_create.assert_called_with(interpreter, options['virtualenv_options'])
 
     def test_create_virtualenv(self):
         env_builder = envbuilder._FadesEnvBuilder()
@@ -225,8 +232,8 @@ class EnvCreationTestCase(unittest.TestCase):
                    "pyvenv_options": [],
                    }
         with patch.object(envbuilder._FadesEnvBuilder, 'create_with_virtualenv') as mock_create:
-                env_builder.create_env(interpreter, is_current, options)
-                mock_create.assert_called_with(interpreter, options['virtualenv_options'])
+            env_builder.create_env(interpreter, is_current, options)
+            mock_create.assert_called_with(interpreter, options['virtualenv_options'])
 
     def test_custom_env_path(self):
         builder = envbuilder._FadesEnvBuilder('some-path')
@@ -361,11 +368,11 @@ class UsageManagerTestCase(unittest.TestCase):
                    "pyvenv_options": ['--system-site-packages'],
                    }
         with patch('fades.envbuilder.helpers.logged_exec') as mock_lexec:
-                # mock_lexec.side_effect = envbuilder.helpers.ExecutionError('matanga!')
-                mock_lexec.side_effect = FileNotFoundError('matanga!')
-                with self.assertRaises(FadesError) as cm:
-                    env_builder.create_env(interpreter, is_current, options)
-                self.assertEqual(str(cm.exception), 'virtualenv not found')
+            # mock_lexec.side_effect = envbuilder.helpers.ExecutionError('matanga!')
+            mock_lexec.side_effect = FileNotFoundError('matanga!')
+            with self.assertRaises(FadesError) as cm:
+                env_builder.create_env(interpreter, is_current, options)
+            self.assertEqual(str(cm.exception), 'virtualenv not found')
 
     def test_executionerror_exception(self):
         env_builder = envbuilder._FadesEnvBuilder()
@@ -375,10 +382,10 @@ class UsageManagerTestCase(unittest.TestCase):
                    "pyvenv_options": ['--system-site-packages'],
                    }
         with patch('fades.envbuilder.helpers.logged_exec') as mock_lexec:
-                mock_lexec.side_effect = envbuilder.helpers.ExecutionError(1, 'cmd', ['stdout'])
-                with self.assertRaises(FadesError) as cm:
-                    env_builder.create_env(interpreter, is_current, options)
-                self.assertEqual(str(cm.exception), 'virtualenv could not be run')
+            mock_lexec.side_effect = envbuilder.helpers.ExecutionError(1, 'cmd', ['stdout'])
+            with self.assertRaises(FadesError) as cm:
+                env_builder.create_env(interpreter, is_current, options)
+            self.assertEqual(str(cm.exception), 'virtualenv could not be run')
 
     def test_general_error_exception(self):
         env_builder = envbuilder._FadesEnvBuilder()
@@ -388,10 +395,10 @@ class UsageManagerTestCase(unittest.TestCase):
                    "pyvenv_options": ['--system-site-packages'],
                    }
         with patch('fades.envbuilder.helpers.logged_exec') as mock_lexec:
-                mock_lexec.side_effect = Exception()
-                with self.assertRaises(FadesError) as cm:
-                    env_builder.create_env(interpreter, is_current, options)
-                self.assertEqual(str(cm.exception), 'General error while running virtualenv')
+            mock_lexec.side_effect = Exception()
+            with self.assertRaises(FadesError) as cm:
+                env_builder.create_env(interpreter, is_current, options)
+            self.assertEqual(str(cm.exception), 'General error while running virtualenv')
 
     def test_when_a_venv_is_removed_it_is_removed_from_everywhere(self):
         old_date = datetime.utcnow()
