@@ -22,7 +22,7 @@ from unittest.mock import patch
 
 from pkg_resources import Requirement
 
-from fades import VERSION, FadesError, __version__, main, parsing
+from fades import VERSION, FadesError, __version__, main, parsing, REPO_PYPI, REPO_VCS
 from tests import create_tempfile
 
 
@@ -167,36 +167,37 @@ class ChildProgramDeciderTestCase(unittest.TestCase):
     """Check how the child program is decided."""
 
     def test_indicated_with_executable_flag(self):
-        analyzable, child = main.decide_child_program(True, "foobar.py")
+        analyzable, child = main.decide_child_program(True, False, "foobar.py")
         self.assertIsNone(analyzable)
         self.assertEqual(child, "foobar.py")
 
     def test_no_child_at_all(self):
-        analyzable, child = main.decide_child_program(False, None)
+        analyzable, child = main.decide_child_program(False, False, None)
         self.assertIsNone(analyzable)
         self.assertIsNone(child)
 
     def test_normal_child_program(self):
         child_path = create_tempfile(self, "")
-        analyzable, child = main.decide_child_program(False, child_path)
+        analyzable, child = main.decide_child_program(False, False, child_path)
         self.assertEqual(analyzable, child_path)
         self.assertEqual(child, child_path)
 
     def test_normal_child_program_not_found(self):
         with self.assertRaises(FadesError):
-            main.decide_child_program(False, 'does_not_exist.py')
+            main.decide_child_program(False, False, 'does_not_exist.py')
 
     def test_normal_child_program_no_access(self):
         child_path = create_tempfile(self, "")
         os.chmod(child_path, 333)  # Remove read permission.
         self.addCleanup(os.chmod, child_path, 644)
         with self.assertRaises(FadesError):
-            main.decide_child_program(False, 'does_not_exist.py')
+            main.decide_child_program(False, False, 'does_not_exist.py')
 
     def test_remote_child_program_simple(self):
         with patch('fades.helpers.download_remote_script') as mock:
             mock.return_value = "new_path_script"
-            analyzable, child = main.decide_child_program(False, "http://scripts.com/foobar.py")
+            analyzable, child = main.decide_child_program(
+                False, False, "http://scripts.com/foobar.py")
             mock.assert_called_with("http://scripts.com/foobar.py")
 
         # check that analyzable and child are the same, and that its content is the remote one
@@ -206,14 +207,105 @@ class ChildProgramDeciderTestCase(unittest.TestCase):
     def test_remote_child_program_ssl(self):
         with patch('fades.helpers.download_remote_script') as mock:
             mock.return_value = "new_path_script"
-            analyzable, child = main.decide_child_program(False, "https://scripts.com/foobar.py")
+            analyzable, child = main.decide_child_program(
+                False, False, "https://scripts.com/foobar.py")
             mock.assert_called_with("https://scripts.com/foobar.py")
 
         # check that analyzable and child are the same, and that its content is the remote one
         self.assertEqual(analyzable, "new_path_script")
         self.assertEqual(child, "new_path_script")
 
-    def test_indicated_with_executable_flag_in_path(self):
-        """Absolute paths not allowed when using --exec."""
+    def test_indicated_with_executable_flag_with_relative_path(self):
+        """Relative paths not allowed when using --exec."""
         with self.assertRaises(FadesError):
-            main.decide_child_program(True, os.path.join("path", "foobar.py"))
+            main.decide_child_program(True, False, os.path.join("path", "../foobar.py"))
+
+    def test_indicated_with_executable_flag_with_absolute_path(self):
+        """Absolute paths are allowed when using --exec."""
+        analyzable, child = main.decide_child_program(True, False, "/tmp/foo/bar.py")
+        self.assertIsNone(analyzable)
+        self.assertEqual(child, "/tmp/foo/bar.py")
+
+    def test_module(self):
+        child_path = 'foo.bar'
+        analyzable, child = main.decide_child_program(False, True, child_path)
+        self.assertIsNone(analyzable)
+        self.assertEqual(child, child_path)
+
+
+# ---------------------------------------
+# autoimport tests
+
+def _autoimport_safe_call(*args, **kwargs):
+    """Call the tested function and always remove the tempfile after the test."""
+    fpath = main.get_autoimport_scriptname(*args, **kwargs)
+
+    with open(fpath, "rt", encoding='utf8') as fh:
+        content = fh.read()
+    os.unlink(fpath)
+
+    return content
+
+
+def test_autoimport_simple():
+    """Simplest autoimport call."""
+    dependencies = {
+        REPO_PYPI: {Requirement.parse('mymod')},
+    }
+    content = _autoimport_safe_call(dependencies, is_ipython=False)
+
+    assert content.startswith(main.AUTOIMPORT_HEADER)
+    assert main.AUTOIMPORT_MOD_IMPORTER.format(module='mymod') in content
+
+
+def test_autoimport_several_dependencies():
+    """Indicate several dependencies."""
+    dependencies = {
+        REPO_PYPI: {Requirement.parse('mymod1'), Requirement.parse('mymod2')},
+    }
+    content = _autoimport_safe_call(dependencies, is_ipython=False)
+
+    assert content.startswith(main.AUTOIMPORT_HEADER)
+    assert main.AUTOIMPORT_MOD_IMPORTER.format(module='mymod1') in content
+    assert main.AUTOIMPORT_MOD_IMPORTER.format(module='mymod2') in content
+
+
+def test_autoimport_including_ipython():
+    """Call with ipython modifier."""
+    dependencies = {
+        REPO_PYPI: {
+            Requirement.parse('mymod'),
+            Requirement.parse('ipython'),  # this one is automatically added
+        },
+    }
+    content = _autoimport_safe_call(dependencies, is_ipython=True)
+
+    assert main.AUTOIMPORT_HEADER not in content
+    assert main.AUTOIMPORT_MOD_IMPORTER.format(module='mymod') in content
+    assert 'ipython' not in content
+
+
+def test_autoimport_no_pypi_dep():
+    """Case with no pypi dependencies."""
+    dependencies = {
+        REPO_PYPI: {Requirement.parse('my_pypi_mod')},
+        REPO_VCS: {'my_vcs_dependency'},
+    }
+    content = _autoimport_safe_call(dependencies, is_ipython=False)
+
+    assert main.AUTOIMPORT_MOD_IMPORTER.format(module='my_pypi_mod') in content
+    assert main.AUTOIMPORT_MOD_SKIPPING.format(dependency='my_vcs_dependency') in content
+
+
+def test_autoimport_importer_mod_ok(capsys):
+    """Check the generated code to import a module when works fine."""
+    code = main.AUTOIMPORT_MOD_IMPORTER.format(module='time')  # something from stdlib, always ok
+    exec(code)
+    assert capsys.readouterr().out == "::fades:: automatically imported 'time'\n"
+
+
+def test_autoimport_importer_mod_fail(capsys):
+    """Check the generated code to import a module when works fine."""
+    code = main.AUTOIMPORT_MOD_IMPORTER.format(module='not_there_should_explode')
+    exec(code)
+    assert capsys.readouterr().out == "::fades:: FAILED to autoimport 'not_there_should_explode'\n"
